/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "Ogre.h"
#include "Platform.h"
#include "Serializer.h"

namespace SLK
{

// Based on http://www.ogre3d.org/wiki/index.php/AnimationBlender
class SerilynExport Animator
{
public:
  enum BlendingTransition
  {
     BlendSwitch,         // stop source and start dest
     BlendWhileAnimating,   // cross fade, blend source animation out while blending destination animation in
     BlendThenAnimate      // blend source to first frame of dest, when done, start dest anim
  };

public: 
	Animator( Ogre::Entity *entity, SLK::Serializer &serializer);
	Animator( Ogre::Entity *entity );
	void update( Ogre::Real );
	void blend( const Ogre::String &animation, BlendingTransition transition, Ogre::Real duration, bool l=true );
	Ogre::Real getProgress() { return m_timeleft/ m_duration; }
	Ogre::AnimationState *getSource() { return m_source; }
	Ogre::AnimationState *getTarget() { return m_target; }
	void init( const Ogre::String &animation, bool loop=true );
	virtual ~Animator() {}

	bool isComplete(){return m_complete;};
	bool isLooping(){return m_source->getLoop();};

	void save(SLK::Serializer &serializer);
private:
	Ogre::AnimationState *loadAnimationIfExists(SLK::Serializer &serializer);
	void saveAnimation(SLK::Serializer &serializer, Ogre::AnimationState *anim);

	Ogre::Entity *m_entity;
	Ogre::AnimationState *m_source;
	Ogre::AnimationState *m_target;

	BlendingTransition m_transition;
	Ogre::Real m_timeleft;
	Ogre::Real m_duration;

	bool m_complete;
};

}