/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "Ogre.h"
#include "Platform.h"
#include "Database.h"

namespace SLK
{

class SerilynExport CameraController3rdPerson
{
public:
	CameraController3rdPerson(Ogre::Camera *camera, Ogre::SceneNode *parentNode, Ogre::Degree yaw=Ogre::Degree(0), Ogre::Degree pitch=Ogre::Degree(0), float zoom=0);
	virtual ~CameraController3rdPerson(void);

	void updateMove(float distance);
	void moveForward(bool move){m_moveForward = move;};
	void moveBackward(bool move){m_moveBackward = move;};
	void moveLeft(bool move){m_moveLeft = move;};
	void moveRight(bool move){m_moveRight = move;};

	void moveZoom(float zoomLevel, float minZoom, float maxZoom);
	void addPitch(Ogre::Degree pitch, Ogre::Degree minPitch, Ogre::Degree maxPitch);
	void addYaw(Ogre::Degree yaw, Ogre::Degree minYaw, Ogre::Degree maxYaw);
	void addPitch(Ogre::Degree pitch);
	void addYaw(Ogre::Degree yaw);

	void setPitch(Ogre::Degree pitch);
	void setYaw(Ogre::Degree yaw);

	Ogre::SceneNode *getBaseNode(){return m_cameraRotate;};
	Ogre::SceneNode *getNode(){return m_cameraNode;};
	void save(SLK::Database *database);
	void load(SLK::Database *database);
private:

	Ogre::Degree m_pitch;
	Ogre::Degree m_yaw;
	Ogre::Camera *m_camera;

	bool m_moveForward;
	bool m_moveBackward;
	bool m_moveLeft;
	bool m_moveRight;

	Ogre::SceneNode *m_cameraRotate;
	Ogre::SceneNode *m_cameraNode;
	Ogre::SceneNode *m_parentNode;
};

}