/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "Ogre.h"
#include "Platform.h"

namespace SLK
{

class SerilynExport Collision
{
public:
	virtual ~Collision(void);

	static inline Ogre::Rect intersect(const Ogre::Rect& lhs, const Ogre::Rect& rhs)
    {
        Ogre::Rect r;

        r.left   = lhs.left   > rhs.left   ? lhs.left   : rhs.left;
        r.top    = lhs.top    > rhs.top    ? lhs.top    : rhs.top;
        r.right  = lhs.right  < rhs.right  ? lhs.right  : rhs.right;
        r.bottom = lhs.bottom < rhs.bottom ? lhs.bottom : rhs.bottom;

        return r;
    }

	static inline bool isDegenerate(const Ogre::Rect& rect)
	{
		return rect.right < rect.left || rect.bottom < rect.top;
	}

	//Returns the final position of the circle
	static Ogre::Vector2 CollideCircleLine(Ogre::Vector2 circlePos, float radius, Ogre::Vector2 lineStart, Ogre::Vector2 lineEnd);
private:
	Collision(void);
};

}