/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef EVENT_H
#define EVENT_H

#include "Platform.h"

namespace SLK
{

class Event;

class EventArgs
{
public:
	EventArgs(){};
	virtual ~EventArgs(){};
};

class SerilynExport EventFunctor
{
public:

	virtual bool operator()(EventArgs &eventArgs)=0;
};

template<typename T>
class EventImpl: public EventFunctor
{
public:
	typedef bool (T::*EventFunction)(EventArgs &eventArgs);

	EventImpl(EventFunction function, T* obj):
        m_function(function),
        m_object(obj)
    {}
	virtual ~EventImpl(void){};

	virtual bool operator()(EventArgs &eventArgs)
    {
        return (m_object->*m_function)(eventArgs);
    }

private:
	EventFunction m_function;
	T* m_object;

};

class SerilynExport Event
{
public:


	template<typename T>
	Event(bool (T::*function)(EventArgs &eventArgs), T* object)
	{
		connect(function, object);
	}

	Event():
		m_impl(0)
	{
	}

	virtual ~Event()
	{
		disconnect();
	}

	bool isConnected(){return m_impl!=0;};

	template<typename T>
	void connect(bool (T::*function)(EventArgs &eventArgs), T* object)
	{
		m_impl = new EventImpl<T>(function,object);
	}
	void disconnect(){SAFEDELETE(m_impl);};

	bool doAction(EventArgs &eventArgs)
	{
		if(isConnected())
			return (*m_impl)(eventArgs);
		else
			return false;
	}

private:
	Event(Event& copy)
	{
		assert(0 && "no copy constructor");
	}

	EventFunctor *m_impl;
};


}

#endif //EVENT_H