/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "random.h"
#include <list>
#include <cassert>
#include <exception>
#include "Platform.h"

namespace SLK
{

template<class T>
class LotteryItem
{
public:
	LotteryItem(T item, int chance)
	{
		m_item = item;
		m_chance = chance;
	};
	T getItem()
	{
		return m_item;
	}
	int getChance()
	{
		return m_chance;
	}
private:
	T m_item;
	int m_chance;
};

template<class T>
class SerilynExport LotteryPicker
{
public:
	LotteryPicker(MTRand &mersenne);
	virtual ~LotteryPicker();
	//Adds an item to the picker with the chance
	void addItem(T item, int chance);

	//True if the picker is empty and false if not
	bool isEmpty();

	//Removes the top item and returns it
	T popItem(){T item = m_items.back().getItem(); m_items.pop_back(); return item;};

	//Picks a random weighted item
	T pick(bool removeAfterPick = false);

	void clear(){m_items.clear(); m_total = 0;};

private:
	std::list<LotteryItem<T> > m_items;
	unsigned int m_total;
	MTRand &m_mersenne;
};

template<class T>
LotteryPicker<T>::LotteryPicker(MTRand &mersenne):
	m_mersenne(mersenne)
{
	m_total=0;
}

template<class T>
LotteryPicker<T>::~LotteryPicker(void)
{
}

template<class T>
void LotteryPicker<T>::addItem(T item, int chance)
{
	LotteryItem<T> pickItem(item,chance);
	m_items.push_back(pickItem);

	m_total+=pickItem.getChance();
}

template<class T>
T LotteryPicker<T>::pick(bool removeAfterPick)
{
	unsigned int pointer=0;	
	unsigned int numPicked = m_mersenne.getBetween(1u,m_total);

	T pickedItem;
	for(std::list<LotteryItem<T> >::iterator iter = m_items.begin(); iter!= m_items.end(); iter++)
	{
		pointer+=(*iter).getChance();
		if(pointer>=numPicked)
		{
			pickedItem = (*iter).getItem();
			if(removeAfterPick)
				m_items.erase(iter);
			return pickedItem;
		}
	}
	assert(0);
	throw std::exception("No items to pick from or bad total");
}


template<class T>
bool LotteryPicker<T>::isEmpty()
{
	return m_items.size()==0;
}

}