/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include <Angelscript.h>
#include <string>
#include <set>

namespace SLK
{

class Scripting;

class Script
{
	friend Scripting;
public:
	void setArg(int arg, asDWORD value){m_scriptContext->SetArgDWord(arg, value);};
	void setArg(int arg, asQWORD value){m_scriptContext->SetArgQWord(arg, value);};
	void setArg(int arg, float value){m_scriptContext->SetArgFloat(arg, value);};
	void setArg(int arg, double value){m_scriptContext->SetArgDouble(arg, value);};
	void setArg(int arg, void* value){m_scriptContext->SetArgObject(arg, value);};

	bool GetReturnBool(){return GetReturnByte() != 0;};
	asBYTE  GetReturnByte(){return m_scriptContext->GetReturnByte();};
	asWORD  GetReturnWord(){return m_scriptContext->GetReturnWord();};
	asDWORD GetReturnDWord(){return m_scriptContext->GetReturnDWord();};
	asQWORD GetReturnQWord(){return m_scriptContext->GetReturnQWord();};
	float   GetReturnFloat(){return m_scriptContext->GetReturnFloat();};
	double  GetReturnDouble(){return m_scriptContext->GetReturnDouble();};
	void   *GetReturnObject(){return m_scriptContext->GetReturnObject();};


	bool prepare(std::string signature = "void main()");
	void* run();

	void prepareAndRun();
private:
	Script(std::string &fileName);
	Script(std::string &id, std::string scriptString);
	virtual ~Script(void);

	bool isOK(){return m_loadOK;};

	void loadScript(const char *file, std::set<std::string> *includedScripts=0);
	int processInclude(std::string &script, std::set<std::string> *includedScripts);

	asIScriptModule *m_scriptModule;
	asIScriptContext *m_scriptContext;
	bool m_loadOK;
};
}
