/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef UIMANAGER_H
#define UIMANAGER_H

#include <ogre.h>
#include <OIS\OIS.h>
#include "CEGUI.h"
#include <CEGUIOgreRenderer.h>
#include "Event.h"
#include "Singleton.h"
#include "HotkeyManager.h"
#include "String.h"

namespace SLK
{

class SerilynExport UIManager: public Singleton<UIManager>,public OIS::KeyListener, public OIS::MouseListener
{
public:
	UIManager(Ogre::RenderWindow *window,Ogre::SceneManager *sceneManager);
	~UIManager(void);

	bool init();
	bool update(float deltaTime);
	CEGUI::Renderer *getRenderer(){return m_guiRenderer;};
	CEGUI::System *getGUISystem(){return m_guiSystem;};
	HotkeyManager *getHotkeyManager(){return m_hotkeyManager;};

	void setWindowExtents( int width, int height );
	inline CEGUI::Window *getMainWindow(){return m_mainWindow;};


private:
	CEGUI::MouseButton convertButton(OIS::MouseButtonID buttonID);

	Ogre::RenderWindow *m_window;
	Ogre::SceneManager *m_sceneManager;

	OIS::Keyboard *m_keyboard;
	OIS::Mouse *m_mouse;
    OIS::InputManager *m_inputManager;

	HotkeyManager *m_hotkeyManager;

	//CEGUI
    CEGUI::Renderer *m_guiRenderer;
    CEGUI::System *m_guiSystem;
	CEGUI::Window *m_mainWindow;

    bool keyPressed( const OIS::KeyEvent &e );
    bool keyReleased( const OIS::KeyEvent &e );

    bool mouseMoved( const OIS::MouseEvent &e );
    bool mousePressed( const OIS::MouseEvent &e, OIS::MouseButtonID id );
    bool mouseReleased( const OIS::MouseEvent &e, OIS::MouseButtonID id );
};

}

#endif //UIManager_H