/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
#include "StdAfx.h"

#include "DebugConsole.h"
namespace SLK
{

DebugConsole::DebugConsole(CEGUI::Window* parent):
	m_multilineScript(false)
{
	m_historyIndex = -1;
	m_historySize = 0;

	CEGUI::WindowManager *winManager = &CEGUI::WindowManager::getSingleton();

	m_screen = winManager->createWindow("DefaultWindow", "DebugConsole");
	parent->addChildWindow(m_screen);
	m_screen->hide();
	m_windowToggler = new WindowToggler(m_screen);

	m_screen->setPosition(CEGUI::UVector2(cegui_absdim(50),cegui_absdim(5)));
	m_screen->setSize(CEGUI::UVector2(cegui_reldim(.3f),cegui_reldim(.5f)));
	m_screen->setAlwaysOnTop(true);

	m_listBox = static_cast<CEGUI::Listbox *>(winManager->createWindow("WindowsLook/Listbox", "DebugConsole/List"));
	m_listBox->setPosition(CEGUI::UVector2(CEGUI::UDim(0,0),CEGUI::UDim(0,0)));
	m_listBox->setSize(CEGUI::UVector2(CEGUI::UDim(1,0),CEGUI::UDim(1,-35.0f)));
	m_screen->addChildWindow(m_listBox);
	m_listBox->setAlpha(0.5f);
	m_listBox->setFont("VeraMono");

	m_inputBox = static_cast<CEGUI::Editbox *>(winManager->createWindow("WindowsLook/Editbox", "DebugConsole/Input"));
	m_inputBox->setPosition(CEGUI::UVector2(CEGUI::UDim(0,0),CEGUI::UDim(1,-30.0f)));
	m_inputBox->setSize(CEGUI::UVector2(CEGUI::UDim(1,0),CEGUI::UDim(0,30.0f)));
	m_screen->addChildWindow(m_inputBox);
	m_inputBox->setAlpha(0.5f);
	m_inputBox->setFont("VeraMono");

	m_screen->subscribeEvent(CEGUI::Window::EventShown,CEGUI::Event::Subscriber(&DebugConsole::onShown,this));

	m_inputBox->subscribeEvent(CEGUI::Editbox::EventKeyDown, CEGUI::Event::Subscriber(&DebugConsole::onTextAdded,this)); 

	Scripting::getSingleton().SetMessageCallback(
		asFUNCTION(LogError),0, asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print(bool)",asFUNCTIONPR(DebugConsole::Print,(bool),void),asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print(int)",asFUNCTIONPR(DebugConsole::Print,(int),void),asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print(uint)",asFUNCTIONPR(DebugConsole::Print,(unsigned int),void),asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print(float)",asFUNCTIONPR(DebugConsole::Print,(float),void),asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print(double)",asFUNCTIONPR(DebugConsole::Print,(double),void),asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print()",asFUNCTIONPR(DebugConsole::Print,(void),void),asCALL_CDECL);
	Scripting::getSingleton().RegisterGlobalFunction("void Print(const string &in)",asFUNCTIONPR(DebugConsole::Print,(const std::string&),void),asCALL_CDECL);	
}

DebugConsole::~DebugConsole(void)
{
	SAFEDELETE(m_windowToggler);
	while(m_listBox->getItemCount()>0)
	{
		CEGUI::ListboxItem *item = m_listBox->getListboxItemFromIndex(0);
		m_listBox->removeItem(item);
	}
}

//adapted from http://www.cegui.org.uk/wiki/index.php/GameChatBox
void DebugConsole::addText(const CEGUI::String& text)
{

	// If there's text then add it
	if(text.size())
	{
		// Add the Editbox text to the history Listbox
		CEGUI::ListboxTextItem* chatItem;
		if(m_listBox->getItemCount() == MAX_HISTORY)
		{
			 //We have reached the capacity of the Listbox so re-use the first ListboxItem.
			 //  This code is a little crafty.  By default the ListboxTextItem is created with
			 //  the auto-delete flag set to true, which results in its automatic deletion when
			 //  removed from the Listbox.  So we change that flag to false, extract the Item
			 //  from the Listbox, change its text, put the auto-delete flag back to true, and
			 //  finally put the Item back into the Listbox. 
			chatItem = static_cast<CEGUI::ListboxTextItem*>(m_listBox->getListboxItemFromIndex(0));
			chatItem->setAutoDeleted(false);
			m_listBox->removeItem(chatItem);
			chatItem->setAutoDeleted(true);
			chatItem->setText(text);
		}
		else
		{
			// Create a new listbox Item
			chatItem = new CEGUI::ListboxTextItem(text);
		}
		m_listBox->addItem(chatItem);
		m_listBox->ensureItemIsVisible(m_listBox->getItemCount());
	}
}	

bool DebugConsole::onTextAdded(const CEGUI::EventArgs& args)
{
	const CEGUI::KeyEventArgs &keyArgs = static_cast<const CEGUI::KeyEventArgs&>(args);

	if(keyArgs.scancode == CEGUI::Key::Return)
	{
		std::string inputString = m_inputBox->getText().c_str();
		if(inputString=="start" && !m_multilineScript)
		{
			m_multilineScript = true;
			addText("**SCRIPT STARTED***");
		}
		else if(inputString=="end" && m_multilineScript)
		{
			m_multilineScript = false;
			addText("**SCRIPT ENDED***");
			Scripting::getSingleton().executeString(m_script);
			m_script = "";
		}
		else if(!m_multilineScript)
		{
			addText(inputString);
			Scripting::getSingleton().executeString(inputString);
		}
		else
		{
			addText(inputString);
			m_script+=std::string(inputString);
		}
		addHistory(inputString);
		m_inputBox->setText("");
		return true;
	}
	else if(keyArgs.scancode == CEGUI::Key::ArrowUp)
	{

		if(m_historyIndex<m_historySize-1)
		{
			m_historyIndex++;
		}
		if(m_historyIndex>=0)
		{
			m_inputBox->setText(m_inputHistory[m_historyIndex]);
		}
		return true;
	}
	else if(keyArgs.scancode == CEGUI::Key::ArrowDown)
	{
		if(m_historyIndex>0)
		{
			m_historyIndex--;
		}
		if(m_historyIndex>=0)
		{
			m_inputBox->setText(m_inputHistory[m_historyIndex]);
		}
		return true;
	}
	
	return false;
}

void DebugConsole::addHistory(std::string text)
{
	for(int i=MAX_INPUT_HISTORY-1; i>0; i--)
	{
		m_inputHistory[i]=m_inputHistory[i-1];
	}

	m_inputHistory[0] = text;

	if(m_historySize<MAX_INPUT_HISTORY)
		m_historySize++;

	m_historyIndex = -1;
}

bool DebugConsole::onShown(const CEGUI::EventArgs& args)
{
	m_inputBox->activate();
	m_inputBox->captureInput();
	return true;
}

}


