#include "StdAfx.h"
#include "MaterialHelper.h"

namespace SLK
{

MaterialHelper::MaterialHelper(void)
{
}

MaterialHelper::~MaterialHelper(void)
{
}

Ogre::MaterialPtr MaterialHelper::createColoredMaterial(std::string materialName, Ogre::ColourValue color)
{
	Ogre::MaterialPtr newMaterial = Ogre::MaterialPtr();

	std::stringstream str;
	str << color.getAsRGBA();
	std::string newMaterialName = materialName + str.str();

	newMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName(newMaterialName);
	if(newMaterial.isNull())
	{
		Ogre::MaterialPtr oldMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName(materialName);

		std::stringstream str;
		str << color.getAsRGBA();
		newMaterial = oldMaterial->clone(newMaterialName,false);

		newMaterial->getBestTechnique()->getPass(0)->setDiffuse(color);
	}

	return newMaterial;
}

Ogre::MaterialPtr MaterialHelper::createTransparentMaterial(std::string materialName, Ogre::ColourValue color)
{
	Ogre::MaterialPtr newMaterial = Ogre::MaterialPtr();

	std::stringstream str;
	str << color.getAsRGBA() << "Alpha";
	std::string newMaterialName = materialName + str.str();

	newMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName(newMaterialName);
	if(newMaterial.isNull())
	{
		Ogre::MaterialPtr oldMaterial = Ogre::MaterialManager::getSingletonPtr()->getByName(materialName);

		newMaterial = oldMaterial->clone(newMaterialName,false);

		newMaterial->getBestTechnique()->getPass(0)->setSceneBlending(Ogre::SBT_TRANSPARENT_ALPHA);
		newMaterial->getBestTechnique()->getPass(0)->setDepthWriteEnabled(false);
		Ogre::ColourValue ambient = newMaterial->getBestTechnique()->getPass(0)->getAmbient();
		Ogre::ColourValue diffuse = newMaterial->getBestTechnique()->getPass(0)->getDiffuse();
		Ogre::ColourValue specular = newMaterial->getBestTechnique()->getPass(0)->getSpecular();
		Ogre::ColourValue emissive = newMaterial->getBestTechnique()->getPass(0)->getSelfIllumination();

		float alpha = 0.5;
		ambient.a = alpha;
		diffuse.a = alpha;
		specular.a = alpha;
		emissive.a = alpha;

		newMaterial->getBestTechnique()->getPass(0)->setAmbient(ambient);
		newMaterial->getBestTechnique()->getPass(0)->setDiffuse(diffuse);
		newMaterial->getBestTechnique()->getPass(0)->setSpecular(specular);
		newMaterial->getBestTechnique()->getPass(0)->setSelfIllumination(emissive);
	}

	return newMaterial;
}

}