#include "StdAfx.h"
#include "MessageBox.h"

namespace SLK
{

unsigned int MessageBox::m_numCtr = 0;
const float MessageBox::ICON_SIZE = 48;
const float MessageBox::BUFFER_SIZE = 20;
const float MessageBox::BUTTON_LENGTH = 64;
const float MessageBox::BUTTON_HEIGHT = 24;
const float MessageBox::DESCRIPTION_HEIGHT = 64;

MessageBox::MessageBox(std::string title, std::string iconName, std::string font, std::string text, std::string button1Text, std::string button2Text, std::string button3Text, std::string button4Text)
{
	CEGUI::WindowManager *winManager = &CEGUI::WindowManager::getSingleton();

	CEGUI::String name = "MessageBox"+CEGUI::PropertyHelper::uintToString(m_numCtr++);
	m_screen = winManager->createWindow("WindowsLook/FrameWindow", name);
	
	m_screen->setText(title);
	m_screen->hide();
	CEGUI::WindowManager::getSingleton().getWindow("Root")->addChildWindow(m_screen);

	CEGUI::Window *titleBar = m_screen->getChild(name+"__auto_titlebar__");
	titleBar->setFont(font);	
	CEGUI::URect titlebarArea= titleBar->getArea();
	CEGUI::UDim titleHeight = titlebarArea.getHeight();
	float absTitleHeight = titleHeight.asAbsolute(titleBar->getParentPixelHeight());

	float width = 5*BUFFER_SIZE+4*BUTTON_LENGTH;
	float height = (int)absTitleHeight+3*BUFFER_SIZE+DESCRIPTION_HEIGHT+BUTTON_HEIGHT;

	m_screen->setWidth(cegui_absdim((float)width));
	m_screen->setHeight(cegui_absdim((float)height));
	m_screen->setProperty("SizingEnabled","False");
	m_screen->setClippedByParent(false);
	m_screen->setProperty("RollUpEnabled","False");
	m_screen->setProperty("CloseButtonEnabled","False");
	m_screen->setProperty("AlwaysOnTop","True");
	m_screen->setRiseOnClickEnabled(true);
	m_screen->setZOrderingEnabled(true);

	m_screen->setPosition(CEGUI::UVector2(cegui_absdim(100),cegui_absdim(100)));



	float descriptionXStart = BUFFER_SIZE;
	if(iconName != "")
	{
		CEGUI::Window *icon = winManager->createWindow("WindowsLook/StaticImage");
		icon->setSize(CEGUI::UVector2(cegui_absdim(ICON_SIZE),cegui_absdim(ICON_SIZE)));
		icon->setPosition(CEGUI::UVector2(cegui_absdim(BUFFER_SIZE),cegui_absdim((absTitleHeight+BUFFER_SIZE))));
		m_screen->addChildWindow(icon);
		icon->setProperty("Image", iconName);
		icon->setProperty("FrameEnabled", "False");
		icon->setProperty("BackgroundEnabled", "False");
		icon->setEnabled(false);

		descriptionXStart += (BUFFER_SIZE + ICON_SIZE);
	}

	float descriptionWidth = width - descriptionXStart - BUFFER_SIZE;
	CEGUI::Window *description = winManager->createWindow("WindowsLook/StaticText");
	description->setSize(CEGUI::UVector2(cegui_absdim(descriptionWidth),cegui_absdim(DESCRIPTION_HEIGHT)));
	description->setPosition(CEGUI::UVector2(cegui_absdim(descriptionXStart),cegui_absdim(absTitleHeight+BUFFER_SIZE)));
	m_screen->addChildWindow(description);
	description->setText(text);
	description->setProperty("FrameEnabled", "False");
	description->setProperty("BackgroundEnabled", "False");
	description->setEnabled(false);
	description->setProperty("HorzFormatting","WordWrapLeftAligned");
	description->setProperty("VertFormatting","TopAligned");

	std::string buttonTexts[4];
	m_numButtons = 1;
	buttonTexts[0] = button1Text;
	if(button2Text != "")
	{
		buttonTexts[1] = button2Text;
		m_numButtons=2;
	}
	if(button3Text != "")
	{
		buttonTexts[2] = button3Text;
		m_numButtons=3;
	}
	if(button4Text !="")
	{
		buttonTexts[3] = button4Text;
		m_numButtons=4;
	}

	float buttonXStart = (width - m_numButtons*BUTTON_LENGTH - (m_numButtons-1)*BUFFER_SIZE)/2;
	
	for(unsigned int i=0; i<m_numButtons; i++)
	{
		m_buttons[i] = winManager->createWindow("WindowsLook/Button");
		
		m_buttons[i]->setSize(CEGUI::UVector2(cegui_absdim(BUTTON_LENGTH),cegui_absdim(BUTTON_HEIGHT)));
		m_buttons[i]->setPosition(CEGUI::UVector2(cegui_absdim(buttonXStart),cegui_absdim(height-BUFFER_SIZE-BUTTON_HEIGHT-30)));
		buttonXStart+= (BUTTON_LENGTH + BUFFER_SIZE);
		
		m_screen->addChildWindow(m_buttons[i]);
		m_buttons[i]->setText(buttonTexts[i]);

		m_buttons[i]->subscribeEvent(CEGUI::ButtonBase::EventActivated,
			CEGUI::Event::Subscriber(&MessageBox::CloseWindow, this));
		m_buttonEvents[i]=0;
	}
	for(unsigned int i=m_numButtons; i<4; i++)
	{
		m_buttons[i]=0;
		m_buttonEvents[i]=0;
	}


}

MessageBox::~MessageBox(void)
{
}

void MessageBox::show()
{
	m_screen->setModalState(true);
	m_screen->moveToFront();
	m_screen->show();
	m_screen->activate();
}

bool MessageBox::CloseWindow(const CEGUI::EventArgs &args)
{
	m_screen->setModalState(false);
	m_screen->hide();

	const CEGUI::WindowEventArgs& wargs = static_cast<const CEGUI::WindowEventArgs&>(args);	
	for(unsigned int i=0; i<m_numButtons; i++)
	{
		if(wargs.window == m_buttons[i])
		{
			EventArgs e;
			if(m_buttonEvents[i])
				m_buttonEvents[i]->doAction(e);
		}
	}

	return true;
}

}
