#include "StdAfx.h"
#include "SelectionHelper.h"
#include "Application.h"

namespace SLK
{

SelectionHelper::SelectionHelper(void)
{
	Ogre::SceneManager *sceneManager = Application::getSingletonPtr()->getSceneManager();
	m_query = sceneManager->createRayQuery(Ogre::Ray());
}

SelectionHelper::~SelectionHelper(void)
{
	Ogre::SceneManager *sceneManager = Application::getSingletonPtr()->getSceneManager();
	sceneManager->destroyQuery(m_query);
}

Ogre::Ray SelectionHelper::getMouseRay()
{
	Ogre::Camera *camera = Application::getSingletonPtr()->getCamera();
	Ogre::Viewport *viewport = Application::getSingletonPtr()->getViewport();
	CEGUI::Point mousePos = CEGUI::MouseCursor::getSingleton().getPosition();
	Ogre::Ray mouseRay = camera->getCameraToViewportRay(mousePos.d_x/viewport->getActualWidth(),mousePos.d_y/viewport->getActualHeight());
	return mouseRay;
}

Ogre::RaySceneQueryResult &SelectionHelper::querySceneAtMouse(Ogre::uint32 mask)
{
	//Show mouseover selection
	Ogre::Ray mouseRay = getMouseRay();
	m_query->setRay(mouseRay);
	return queryScene(mask);
}


Ogre::RaySceneQueryResult &SelectionHelper::queryScene(Ogre::uint32 mask)
{
	m_query->setQueryMask(mask);
	Ogre::RaySceneQueryResult &result = m_query->execute();

	return result;
}


Ogre::MovableObject *SelectionHelper::getClickedMovable(int flags)
{
	Ogre::RaySceneQueryResult &result = querySceneAtMouse(flags);
	if(result.size() == 0)
		return 0;
	else
	{
		return result[0].movable;
	}
}

bool SelectionHelper::getPlaneIntersect(Ogre::Vector3 &pos, float height)
{
	Ogre::Ray mouseRay = getMouseRay();

	std::pair<bool, Ogre::Real> pair = mouseRay.intersects(Ogre::Plane(Ogre::Vector3(0,-1,0),Ogre::Vector3(0,height,0)));
	pos = mouseRay.getPoint(pair.second);

	return pair.first;
}



}